package de.hdm_stuttgart.mi.sd1.carjump.model;

/**
 * Model of a car jump.
 *
 */
public class CarJump {

   // Physical constant describing a fall's acceleration 
   // due to gravity.
   private static final double gravitationalAcceleration = 9.81;

   // Model parameters
   private double
     accelHeight,
     slopeHeight;

   // derived parameters
   double
     slopeSpeed;

   /**
    * Internal method to be called at end of object construction
    * and after changing any model parameter. Re-calculates
    * derived parameters, see above.
    */
   private void calculateDerivedParameters() {
      slopeSpeed = Math.sqrt(2 * gravitationalAcceleration * (accelHeight - slopeHeight));
   }

   /**
    * A model of a car gaining speed and then getting airborne for a jump between
    * two slopes. 
    * 
    * @param accelHeight See {@link #setAccelHeight(double)}
    * @param slopeHeight  See {@link #setSlopeHeight(double)}
    */
   public CarJump(final double accelHeight, final double slopeHeight) {
      // We assign attribute values since calling i.e. setAccelHeight(...) would trigger the invocation
      // of calculateDerivedParameters(). But this invocation must be deferred till our object
      // has been fully initialized.
      //
      this.accelHeight = accelHeight;
      this.slopeHeight = slopeHeight;
      calculateDerivedParameters();
   }

   /**
    * @return The car's speed when leaving the ascending slope.
    */
   public double getSlopeSpeed() {
      return slopeSpeed;
   }
   
   /**
    * @param accelHeight The height measured in meters at which the car will start at the very beginning of its journey.
    */
   public void setAccelHeight(double accelHeight) {
      if (accelHeight != this.accelHeight) { // Only assign and re-calculate if required
         this.accelHeight = accelHeight;
         calculateDerivedParameters();
      }
   }
   
   /**
    * @param slopeHeight The slope's tip height above ground level measured in meters. 
    */
   public void setSlopeHeight(double slopeHeight) {
      if (slopeHeight != this.slopeHeight) {// Only assign and re-calculate if required
         this.slopeHeight = slopeHeight;
         calculateDerivedParameters();
      }
   }
}